@@ -2,26 +2,25 @@
  * Gnome Chemistry Utils
  * object.cc 
  *
- * Copyright (C) 2002-2004
+ * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * This program is free software; you can redistribute it and/or 
+ * modify it under the terms of the GNU General Public License as 
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the 
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA  02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
+ * USA
  */
 
+#include "config.h"
 #include "object.h"
 #include "document.h"
 #include <string>
@@ -312,13 +311,20 @@ void Object::Transform2D(Matrix2D& m, double x, double y)
 	for (i = m_Children.begin(); i != m_Children.end(); i++) (*i).second->Transform2D(m, x, y);
 }
 
-bool Object::BuildContextualMenu (GtkUIManager *UIManager, Object *object)
+bool Object::BuildContextualMenu (GtkUIManager *UIManager, Object *object, double x, double y)
 {
-	return (m_Parent)? m_Parent->BuildContextualMenu (UIManager, object): false;
+	return (m_Parent)? m_Parent->BuildContextualMenu (UIManager, object, x, y): false;
 }
 
-void Object::Add(GtkWidget* w)
+void Object::Add (GtkWidget* w)
 {
+	map<string, Object*>::iterator i;
+	Object* p = GetFirstChild (i);
+	while (p)
+	{
+		p->Add (w);
+		p = GetNextChild (i);
+	}
 }
 
 void Object::Print(GnomePrintContext *pc)
@@ -386,19 +392,19 @@ TypeId Object::AddType(string TypeName, Object*(*Create)(), TypeId id)
 		NextType = TypeId ((unsigned) NextType + 1);
 	} else
 		typedesc.Id = id;
-	if (TypeNames.capacity() <=  typedesc.Id) {
-		size_t max = (((size_t)  typedesc.Id / 10) + 1) * 10;
-		TypeNames.reserve (max);
-		while (max > TypeNames.size())
-			TypeNames.push_back ("");
+	if (TypeNames.size() <= typedesc.Id) {
+		size_t max = (((size_t) typedesc.Id / 10) + 1) * 10;
+		TypeNames.resize (max);
 	}
 #if HAS_VECTOR_AT
-	TypeNames.at (id) = TypeName;
+	TypeNames.at (typedesc.Id) = TypeName;
 #else
 	vector<string>::iterator it;
 	it = TypeNames.begin ();
 	it += typedesc.Id;
-	TypeNames.insert (it, TypeName);
+	it = TypeNames.insert (it, TypeName);
+	it++;
+	TypeNames.erase (it);
 #endif
 	return typedesc.Id;
 }
